/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	import com.boostworthy.geom.Path;
	import com.dncompute.graphics.*;
	import flash.display.*;
	import flash.events.*;
	import flash.geom.Rectangle;
	import flash.text.*;
	import flash.ui.*;
	import com.boostworthy.animation.management.*;
	import com.boostworthy.animation.easing.*;
	import com.boostworthy.animation.sequence.*;
	import com.boostworthy.animation.sequence.tweens.*;
	import com.boostworthy.animation.rendering.*;
	import com.boostworthy.core.Global;
	
	/**
	 * Animation for DMP serial explanation.
	 * @author Joe Devietti
	 */
	public class Serial extends Sprite {
		
	public function Serial() {
		if (stage) init();
		else addEventListener(Event.ADDED_TO_STAGE, init);
	}
	
	private const g_timeline:Timeline = new Timeline( RenderMethod.ENTER_FRAME );
	
	private function init(e:Event = null):void {
		removeEventListener(Event.ADDED_TO_STAGE, init);
		// entry point
		
		SceneManager.setup( g_timeline, stage );
		
		//trace( "main Sprite's name is: " + this.name );
		
		// SHOW CODE + CPU0
		var sp_scene0_sprites:Array = [];
		var sp_labels:Array = [];
		
		// draw codebox
		var s:Sprite = MyShapes.CodeBox( ExecLayout.INSNS[i] );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CODE_GAP;
		s.y = ExecLayout.CODE_TOP;
		sp_scene0_sprites.push( s );
		addChild( s );
		
		// draw codebox label
		var label:Sprite = MyShapes.Label( s, "Code", MyShapes.LABEL_RIGHT );
		label.x = s.x + s.width + 10;
		label.y = s.y;
		addChild( label );
		sp_labels.push( label );
		
		// draw cpu
		var i:int = 0;
		s = MyShapes.Cpu( "p"+i );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CPU_GAP;
		s.y = ExecLayout.CPU_TOP;
		sp_scene0_sprites.push( s );
		addChild( s );
		
		// draw cpu label
		label = MyShapes.Label( s, "Processor", MyShapes.LABEL_RIGHT );
		label.x = s.x + s.width + 10;
		label.y = s.y;
		addChild( label );
		sp_labels.push( label );

		
// SHOW REST OF CODE AND CPUS
var scene2_sprites:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// local iteration vars
	var i:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	
	// fade out labels
	for each ( s in sp_labels ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,0.0) );
	}
	Anim.StopIncr(Anim.FADE);
	
	for ( i = 1; i < ExecLayout.NUM_CPUS; i++ ) {
		// draw codebox
		s = MyShapes.CodeBox( ExecLayout.INSNS[i] );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CODE_GAP;
		s.y = ExecLayout.CODE_TOP;
		s.alpha = 0.0;
		addChild( s );
		scene2_sprites.push( s );
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
		
		// draw cpu
		s = MyShapes.Cpu( "p"+i );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CPU_GAP;
		s.y = ExecLayout.CPU_TOP;
		s.alpha = 0.0;
		addChild( s );
		scene2_sprites.push( s );
		
		g_timeline.addTween( ExecLayout.fadeToIncr(s,1.0) );
	}
},
function():void {
	for each ( var s:Sprite in scene2_sprites ) { removeChild(s); }
	scene2_sprites = [];
},
3*Anim.FADE);


// SHOW TIME ARROWS
var sp_arrows:Array = [];
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	for ( var i:int = 1; i <= 2; i++ ) {
		var s:Sprite = MyShapes.TimeArrow();
		s.x = ExecLayout.COLUMN_MIDPOINT_START[i] + (s.width/2);
		s.y = ExecLayout.INSN_TOP;
		addChild( s );
		sp_arrows.push( s );
		
		// wipe in time arrows - have to use a path wipe because the regular wipe 
		// assumes (0,0) as the TL corner
		var path:Path = new Path();
		path.moveTo( 0, 0 );
		/** NB: path is in arrow's coord space (unrotated), so we specify a downward 
		 * wipe of the height of arrow by using s.height (rotated stage coords)
		 * as the x value. */
		path.lineTo( s.height, 0 );
		g_timeline.addTween( new PathTween( Wiper.pathWipe(s,path), path, true, Anim.Start(),
			Anim.StopNoIncr(Anim.TIME_ARROW_WIPE), Transitions.LINEAR ) );
	}
	Anim.StopIncr(Anim.TIME_ARROW_WIPE);
},
function():void {
	for each ( var s:Sprite in sp_arrows ) { removeChild( s ); }
	sp_arrows = [];
},
Anim.TIME_ARROW_WIPE);


// ENTER THE TOKEN
var sp_token:Sprite = null;
var sp_token_label:Sprite = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	
	// dim all visible sprites
	for each ( s in scene2_sprites.concat(sp_scene0_sprites,sp_arrows) ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,ExecLayout.ALPHA_DIM) );
	}
	// dim shapes at same time as we draw the token
	
	// draw token at p0
	sp_token = MyShapes.Token();
	sp_token.x = ExecLayout.tokenAt(0).x;
	sp_token.y = ExecLayout.tokenAt(0).y;
	sp_token.alpha = 0.0;
	addChild( sp_token );
	
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_token,1.0) );
	
	// draw token label
	sp_token_label = MyShapes.Label( sp_token, "Deterministic Token", MyShapes.LABEL_RIGHT );
	sp_token_label.x = sp_token.x + sp_token.width + 10;
	sp_token_label.y = sp_token.y;
	sp_token_label.alpha = 0.0;
	addChild( sp_token_label );
	
	ExecLayout.g_current_token_cpu = 0;
	
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_token_label,1.0) );
},
function():void {
	removeChild( sp_token );
	sp_token = null;
	removeChild( sp_token_label );
	sp_token_label = null;
},
Anim.FADE*2); 


// ADD DESCRIPTIVE BULLETS
var sp_token_bullets:DisplayObject = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	var txt:TextField = new UnselectableTextField();
	txt.htmlText = "<li>token visits processors in deterministic round robin order</li>\n<li>each processor executes 1 instruction and then passes token</li>";
	txt.autoSize = TextFieldAutoSize.LEFT;
	txt.wordWrap = true;
	txt.width = 500;
	txt.setTextFormat( MyShapes.getLabelTextFormat() );
	addChild( txt );
	txt.x = 50;
	txt.y = 150;
	txt.alpha = 0;
	
	sp_token_bullets = txt;
	g_timeline.addTween( ExecLayout.fadeToIncr(txt,1.0) );
},
function():void {
	removeChild( sp_token_bullets );
	sp_token_bullets = null;
},
Anim.FADE);


// REMOVE TOKEN LABEL
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	
	// make sprites visible again
	for each ( s in scene2_sprites.concat(sp_scene0_sprites,sp_arrows) ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	}
	
	// fade out token label and bullets
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_token_label,0.0) );
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_token_bullets,0.0) );

	
	Anim.StopIncr(Anim.FADE);
},
function():void {},
Anim.FADE);


// SHOW 1-AT-A-TIME INSN EXECUTION
var sp_1aat_insns:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// local iteration vars
	var i:int = 0; var j:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	
	var insn_voffset:int = ExecLayout.INSN_TOP;
	
	for ( j = 0; j < ExecLayout.INSNS_PER_CPU; j++ ) {
		for ( i = 0; i < ExecLayout.NUM_CPUS; i++ ) {
			
			// move token to next proc, but not for first insn (it's already at p0)
			if ( 0 != i || 0 != j ) {
				g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,i) );
			}
			
			s = MyShapes.Insn( ExecLayout.INSNS[i][j], MyShapes.INSN_COLORS[i] );
			s.x = ExecLayout.COLUMN_START[i] + ExecLayout.INSN_GAP;
			s.y = insn_voffset;
			s.alpha = 0.0;
			insn_voffset += MyShapes.INSN_YOFFSET;
			
			addChild( s );
			sp_1aat_insns.push( s );
			
			g_timeline.addTween( ExecLayout.fadeToIncr(s,1.0) );
			
		}
	}
},
function():void {
	for each ( var s:Sprite in sp_1aat_insns ) { removeChild(s); }
	sp_1aat_insns = [];
},
(6*Anim.INSN_APPEAR)+(5*Anim.MOVE_TOKEN) );


// SETUP FOR QUANTUM EXECUTION
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// move token to p0
	g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,0) );
	
	// fade 1-at-a-time execution
	for each ( var s:Sprite in sp_1aat_insns ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,0.0) );
	}
	Anim.StopIncr(Anim.FADE);
},
function():void {},
Anim.MOVE_TOKEN+Anim.FADE);


// SHOW QUANTUM EXECUTION
var sp_quantum_insns:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// local iteration vars
	var i:int = 0; var j:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	
	var insn_voffset:int = ExecLayout.INSN_TOP;
	
	for ( i = 0; i < ExecLayout.NUM_CPUS; i++ ) {
		// move token to next proc, (except for first quantum, as token is already at p0)
		if ( 0 != i ) {
			g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,i) );
		}
		
		// show insns
		for ( j = 0; j < ExecLayout.INSNS_PER_CPU; j++ ) {
			
			s = MyShapes.Insn( ExecLayout.INSNS[i][j], MyShapes.INSN_COLORS[i] );
			s.x = ExecLayout.COLUMN_START[i] + ExecLayout.INSN_GAP;
			s.y = insn_voffset;
			s.alpha = 0.0;
			s.name = ("i"+i)+j;
			insn_voffset += MyShapes.INSN_YOFFSET;
			
			addChild( s );
			sp_quantum_insns.push( s );
			
			g_timeline.addTween( ExecLayout.fadeToIncr(s,1.0) );
		}
	}
},
function():void {
	for each ( var s:Sprite in sp_quantum_insns ) { removeChild(s); }
	sp_quantum_insns = [];
},
(6*Anim.INSN_APPEAR)+(2*Anim.MOVE_TOKEN) );


// LABEL A QUANTUM
var sp_quantum_label:Sprite = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	
	// need a fake display object the size of 2 insns
	var box:Sprite = new Sprite();
	box.graphics.lineStyle( 1 );
	box.graphics.drawRect( 0, 0, MyShapes.INSN_WIDTH, MyShapes.INSN_HEIGHT+MyShapes.INSN_YOFFSET ); // NB: padding b/w insns
	
	// draw quantum label
	sp_quantum_label = MyShapes.Label( box, "Quantum", MyShapes.LABEL_RIGHT );
	sp_quantum_label.x = ExecLayout.COLUMN_START[1] + ExecLayout.INSN_GAP + MyShapes.INSN_WIDTH + MyShapes.LABEL_OFFSET;
	sp_quantum_label.y = ExecLayout.INSN_TOP + (2*MyShapes.INSN_YOFFSET);
	sp_quantum_label.alpha = 0.0;
	addChild( sp_quantum_label );
	
	// dim all visible objects
	for each ( s in scene2_sprites.concat(sp_scene0_sprites,sp_arrows,sp_token,sp_quantum_insns) ) {
		if ( s.name != "i10" && s.name != "i11" ) {
			g_timeline.addTween( ExecLayout.fadeToNoIncr(s,ExecLayout.ALPHA_DIM) );
		}
	}
	Anim.StopIncr(Anim.FADE);
	
	// highlight label
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_quantum_label,1.0) );
},
function():void {
	removeChild( sp_quantum_label );
	sp_quantum_label = null;
},
Anim.FADE*2);


// SHOW COMMUNICATION ARROWS
var sp_comm_arrows:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// dim everything that's not insns
	for each ( var s:Sprite in sp_scene0_sprites ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,ExecLayout.ALPHA_DIM) );
	}
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_quantum_label,0.0) );
	// re-highlight all quantum insns
	for each ( s in sp_quantum_insns ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	}
	Anim.StopIncr(Anim.FADE);
	
	// wipe in communication arrows
	var from:Sprite = ExecLayout.findByName( sp_quantum_insns, "i00" );
	var to:Sprite = ExecLayout.findByName( sp_quantum_insns, "i11" );
	var arrow:Sprite = new TextLineArrow( from.x+from.width - ExecLayout.COMM_ARROW_XOFFSET, from.y+(from.height/2), 
		to.x + ExecLayout.COMM_ARROW_XOFFSET, to.y+(to.height/2), "RAW", ExecLayout.COMM_ARROW_COLOR );
	addChild( arrow );
	sp_comm_arrows.push( arrow );
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(arrow, Anim.COMM_ARROW_WIPE) );

	from = ExecLayout.findByName( sp_quantum_insns, "i10" );
	to = ExecLayout.findByName( sp_quantum_insns, "i21" );
	arrow = new TextLineArrow( from.x+from.width - ExecLayout.COMM_ARROW_XOFFSET, from.y+(from.height/2),
		to.x + ExecLayout.COMM_ARROW_XOFFSET, to.y+(to.height/2), "WAW", ExecLayout.COMM_ARROW_COLOR );
	addChild( arrow );
	sp_comm_arrows.push( arrow );
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(arrow, Anim.COMM_ARROW_WIPE) );	
},
function():void {
	for each ( var s:Sprite in sp_comm_arrows ) { removeChild(s); }
	sp_comm_arrows = [];
},
(Anim.FADE) + (2*Anim.COMM_ARROW_WIPE));


		SceneManager.noMoreScenes();
	} // end init()
	
}}