/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	import com.boostworthy.geom.Path;
	import com.dncompute.graphics.ArrowStyle;
	import com.dncompute.graphics.LineArrow;
	import flash.display.*;
	import flash.events.*;
	import flash.ui.*;
	import com.boostworthy.animation.management.*;
	import com.boostworthy.animation.easing.*;
	import com.boostworthy.animation.sequence.*;
	import com.boostworthy.animation.sequence.tweens.*;
	import com.boostworthy.animation.rendering.*;
	import com.boostworthy.core.Global;
	
	/**
	 * Animation describing the Sharing Table abstractly in terms of parallel prefixes
	 * and serial suffixes.
	 * @author Joe Devietti
	 */
	public class ShtabConcept extends Sprite {
		
	private const STAGE_WIDTH:uint = 500;
	private const NUM_QUANTA:uint = 3;
	private const COLUMN_WIDTH:Number = STAGE_WIDTH / NUM_QUANTA;
	private const COLUMN_START:Array = [ 0, COLUMN_WIDTH, 2 * COLUMN_WIDTH ];
	
	private const QUANTUM_TOP:uint = 0;
	private const QUANTUM_GAP:Number = (COLUMN_WIDTH - MyShapes.QUANTUM_WIDTH) / 2;

	private const g_timeline:Timeline = new Timeline( RenderMethod.ENTER_FRAME );
		
	public function ShtabConcept() {
		if (stage) init();
		else addEventListener(Event.ADDED_TO_STAGE, init);
	}
	
	private function init(e:Event = null):void {
		removeEventListener(Event.ADDED_TO_STAGE, init);
		// entry point
		
		SceneManager.setup( g_timeline, stage );
		//SceneManager.setup( this.animate, g_timeline, stage );
		
	//}	
	//public function animate():void {
	
// SHOW SERIALIZED QUANTA
var sp_quanta:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// local iteration vars
	var i:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	
	for ( i = 0; i < NUM_QUANTA; i++ ) {
		
		// draw quantum
		s = MyShapes.Quantum();
		s.x = COLUMN_START[i] + QUANTUM_GAP;
		s.y = QUANTUM_TOP + (i * (2*MyShapes.QUANTUM_HEIGHT));
		s.alpha = 0.0;
		s.name = "q"+i;
		addChild( s );
		sp_quanta.push( s );
		g_timeline.addTween( new Tween( s, "alpha", 1.0, Anim.Start(), Anim.StopIncr(Anim.FADE), Transitions.LINEAR) );
		
	}
},
function():void {
	for each ( var s:Sprite in sp_quanta ) { removeChild(s); }
	sp_quanta = [];
},
Anim.FADE*3);


SceneManager.addPowerpointClick();


// BREAK QUANTA INTO PARALLEL/SERIAL *FIXES
var sp_fixes:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// local iteration vars
	var i:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	
	for ( i = 0; i < NUM_QUANTA; i++ ) {
		
		// draw parallel prefix
		s = MyShapes.ParallelPrefix();
		s.x = COLUMN_START[i] + QUANTUM_GAP;
		s.y = QUANTUM_TOP + (i * (2*MyShapes.QUANTUM_HEIGHT));
		s.alpha = 0.0;
		s.name = "p"+i;
		addChild( s );
		sp_fixes.push( s );
		g_timeline.addTween( new Tween( s, "alpha", 1.0, Anim.Start(),
			Anim.StopNoIncr(Anim.FADE), Transitions.LINEAR) );
		
		// draw serial suffix
		s = MyShapes.SerialSuffix();
		s.x = COLUMN_START[i] + QUANTUM_GAP;
		s.y = QUANTUM_TOP + (i * (2*MyShapes.QUANTUM_HEIGHT)) + MyShapes.QUANTUM_HEIGHT;
		s.alpha = 0.0;
		s.name = "s"+i;
		addChild( s );
		sp_fixes.push( s );
		g_timeline.addTween( new Tween( s, "alpha", 1.0, Anim.Start(),
			Anim.StopIncr(Anim.FADE), Transitions.LINEAR) );
	}
	//Anim.StopIncr(Anim.FADE);
},
function():void {
	for each ( var s:Sprite in sp_fixes ) { removeChild(s); }
	sp_fixes = [];
},
Anim.FADE*3);


// DIM SERIALIZED QUANTA, SLIDE *FIXES UP
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// local iteration vars
	var i:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	
	for each ( s in sp_fixes ) {
		if ( s.name.charAt(0) == "p" ) {
			i = int(s.name.charAt(1));
			g_timeline.addTween( new Tween(s, "y", s.y - (i * 2 * MyShapes.QUANTUM_HEIGHT), 
				Anim.Start(), Anim.StopNoIncr(Anim.MOVE_QUANTUM), Transitions.CUBIC_IN_AND_OUT) );
			
		} else { // serial suffix
			i = int(s.name.charAt(1));
			g_timeline.addTween( new Tween(s, "y", s.y - (i * MyShapes.QUANTUM_HEIGHT), 
				Anim.Start(), Anim.StopNoIncr(Anim.MOVE_QUANTUM), Transitions.CUBIC_IN_AND_OUT) );
		}
	}
	Anim.StopIncr(Anim.MOVE_QUANTUM);
	for each ( s in sp_quanta ) {
		s.alpha = 0.3;
	}
},
function():void {},
Anim.MOVE_QUANTUM);
		
		SceneManager.noMoreScenes();
	} // end init()
	
}}