/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	import com.boostworthy.geom.Path;
	import com.dncompute.graphics.*;
	import flash.display.*;
	import flash.events.*;
	import flash.text.*;
	import flash.geom.Point;
	import flash.ui.*;
	import com.boostworthy.animation.management.*;
	import com.boostworthy.animation.easing.*;
	import com.boostworthy.animation.sequence.*;
	import com.boostworthy.animation.sequence.tweens.*;
	import com.boostworthy.animation.rendering.*;
	import com.boostworthy.core.Global;
	
	/**
	 * Animation for DMP serial explanation.
	 * @author Joe Devietti
	 */
	public class TMForward extends Sprite {
		
	public function TMForward() {
		if (stage) init();
		else addEventListener(Event.ADDED_TO_STAGE, init);
	}
	
	private const g_timeline:Timeline = new Timeline( RenderMethod.ENTER_FRAME );
	
	private function ForwardingArrow(x1:int,y1:int,x2:int,y2:int):Sprite {
		return new TextLineArrow( x1-ExecLayout.COMM_ARROW_XOFFSET, y1, 
			x2+ExecLayout.COMM_ARROW_XOFFSET, y2, "FORWARD", 0x3399CC, 0x3399CC, 0xFFFFFF );
	}
	
	private function init(e:Event = null):void {
		removeEventListener(Event.ADDED_TO_STAGE, init);
		// entry point
		
		SceneManager.setup( g_timeline, stage );
		


// SHOW CODE, CPUS AND TOKEN
	// local iteration vars
	var i:int = 0; var j:int = 0;
	var s:Sprite = null; var t:Sprite = null;
	var w:Object = null;

	var sp_code_cpus:Array = [];
	for ( i = 0; i < ExecLayout.NUM_CPUS; i++ ) {
		// draw codebox
		s = MyShapes.CodeBox( ExecLayout.INSNS[i] );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CODE_GAP;
		s.y = ExecLayout.CODE_TOP;
		addChild( s );
		sp_code_cpus.push( s );
		
		// draw cpu
		s = MyShapes.Cpu( "p"+i );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CPU_GAP;
		s.y = ExecLayout.CPU_TOP;
		addChild( s );
		sp_code_cpus.push( s );
	}
	
	// draw token at p0
	var sp_token:Sprite = MyShapes.Token();
	sp_token.x = ExecLayout.tokenAt(0).x;
	sp_token.y = ExecLayout.tokenAt(0).y;
	ExecLayout.g_current_token_cpu = 0;
	addChild( sp_token );
	
	// time arrows
	var sp_time_arrows:Array = [];
	for ( i = 1; i <= 2; i++ ) {
		s = MyShapes.TimeArrow();
		s.x = ExecLayout.COLUMN_MIDPOINT_START[i] + (s.width/2);
		s.y = ExecLayout.INSN_TOP;
		addChild( s );
		sp_time_arrows.push( s );
	}

	// draw xact boxes
	var sp_quanta:Array = [];
	for ( i = 0; i < ExecLayout.NUM_CPUS; i++ ) {
		var xact:Sprite = MyShapes.Xact(2);
		xact.x = ExecLayout.COLUMN_START[i] + ExecLayout.XACT_GAP;
		xact.y = ExecLayout.XACT_TOP;
		xact.name = "xact"+i;
		addChild( xact );
		sp_quanta.push( xact );
	}
	
	// draw insns
	var sp_insns:Array = [];
	for ( j = 0; j < ExecLayout.INSNS_PER_CPU; j++ ) {
		for ( i = 0; i < ExecLayout.NUM_CPUS; i++ ) {
			s = MyShapes.Insn( ExecLayout.INSNS[i][j], MyShapes.INSN_COLORS[i] );
			s.name = ("i"+i)+j;
			sp_insns.push( s );
			addChild( s );
			
			s.x = ExecLayout.COLUMN_START[i] + ExecLayout.XACT_GAP + MyShapes.XACT_XOFFSET;
			s.y = ExecLayout.XACT_TOP + MyShapes.XACT_YOFFSET + (j*MyShapes.INSN_YOFFSET);
		}
	}
	
	// commit xact0
	var sp_commits:Array = [];
	var commit:Sprite = MyShapes.XactCommit();
	commit.x = ExecLayout.COLUMN_START[0] + ExecLayout.XACT_GAP;
	
	xact = ExecLayout.findByName( sp_quanta, "xact0" );
	commit.y = xact.y + xact.height - MyShapes.XACT_LINE_THICKNESS;
	
	commit.name = "commit0";
	addChild( commit );
	sp_commits.push( commit );

	
// SHOW CONFLICT ARROW
var sp_conflict_arrow01:Sprite = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// draw conflict arrow
	var from:Sprite = ExecLayout.findByName( sp_insns, "i00" );
	var fromP:Point = from.localToGlobal( new Point(from.width,(from.height/2)) );
	var to:Sprite = ExecLayout.findByName( sp_insns, "i11" );
	var toP:Point = to.localToGlobal( new Point(0,(to.height/2)) );
	sp_conflict_arrow01 = MyShapes.ConflictArrow( fromP.x, fromP.y, toP.x, toP.y, "CONFLICT" );
	addChild( sp_conflict_arrow01 );
	
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(sp_conflict_arrow01, Anim.COMM_ARROW_WIPE) );
},
function():void {
	removeChild( sp_conflict_arrow01 );
	sp_conflict_arrow01 = null;
},
Anim.COMM_ARROW_WIPE);


// REPLACE CONFLICT WITH FORWARDING BETWEEN XACT0 AND XACT1
var sp_forwarding:Sprite = null;
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	// draw conflict arrow
	var from:Sprite = ExecLayout.findByName( sp_insns, "i00" );
	var to:Sprite = ExecLayout.findByName( sp_insns, "i11" );
	sp_forwarding = ForwardingArrow( from.x+from.width, from.y+(from.height/2), to.x, to.y+(to.height/2) );	
	addChild( sp_forwarding );
	
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(sp_forwarding, Anim.COMM_ARROW_WIPE) );
	
	// fade out conflict arrow
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_conflict_arrow01,0.0) );
},
function():void {
	removeChild( sp_forwarding );
	sp_forwarding = null;
},
Anim.COMM_ARROW_WIPE);


// PASS TOKEN TO P1
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,1) );
},
function():void {},
Anim.MOVE_TOKEN);


// TODO: show a little checkmark to "verify" the forwarding before commit?


// EXTEND XACT1 AND COMMIT
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	var old_xact:Sprite = ExecLayout.findByName( sp_quanta, "xact1" );
	
	// draw new xact1
	i = 1;
	var new_xact:Sprite = MyShapes.Xact(2.2);
	var new_xact_height:int = new_xact.height; // NB: need to save this here as adding mask changes height!
	new_xact.x = old_xact.x;
	new_xact.y = old_xact.y;
	new_xact.name = "xact"+i+"b";
	addChildAt( new_xact, 0 ); // draw below forwarding arrow
	sp_quanta.push( new_xact );
	
	// wipe in larger xact
	w = Wiper.wipe( new_xact, Wiper.WIPE_DOWN, true );
	g_timeline.addTween( new Tween(w.mask,"y",w.y,Anim.Start(),Anim.StopNoIncr(Anim.XACT_COMMIT),Transitions.LINEAR) );
	
	// wipe out old xact to avoid overlap
	var stop:int = Anim.Start() + ((old_xact.height / new_xact_height) * Anim.XACT_COMMIT);
	w = Wiper.wipe( old_xact, Wiper.WIPE_DOWN, false );
	g_timeline.addTween( new Tween(w.mask,"y",w.y,Anim.Start(),stop,Transitions.LINEAR) );
	Anim.StopIncr(Anim.XACT_COMMIT);
	
	// commit xact1
	var commit:Sprite = MyShapes.XactCommit();
	commit.x = ExecLayout.COLUMN_START[i] + ExecLayout.XACT_GAP;
	commit.y = new_xact.y + new_xact_height - MyShapes.XACT_LINE_THICKNESS;

	commit.name = "commit"+i;
	addChild( commit );
	sp_commits.push( commit );
	w = Wiper.wipe( commit, Wiper.WIPE_RIGHT, true );
	g_timeline.addTween( new Tween(w.mask,"x",w.x,Anim.Start(),Anim.StopIncr(Anim.XACT_COMMIT),Transitions.LINEAR) );

	
	// pass token to p2
	g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,2) );
	
	old_xact = ExecLayout.findByName( sp_quanta, "xact2" );
	
	// draw new xact2
	i = 2;
	new_xact = MyShapes.Xact(2.4);
	new_xact_height = new_xact.height; // NB: need to save this here as adding mask changes height!
	new_xact.x = old_xact.x;
	new_xact.y = old_xact.y;
	new_xact.name = "xact"+i+"b";
	addChildAt( new_xact, 0 ); // draw below forwarding arrow
	sp_quanta.push( new_xact );
	
	// wipe in larger xact
	w = Wiper.wipe( new_xact, Wiper.WIPE_DOWN, true );
	g_timeline.addTween( new Tween(w.mask,"y",w.y,Anim.Start(),Anim.StopNoIncr(Anim.XACT_COMMIT),Transitions.LINEAR) );
	
	// wipe out old xact to avoid overlap
	stop = Anim.Start() + ((old_xact.height / new_xact_height) * Anim.XACT_COMMIT);
	w = Wiper.wipe( old_xact, Wiper.WIPE_DOWN, false );
	g_timeline.addTween( new Tween(w.mask,"y",w.y,Anim.Start(),stop,Transitions.LINEAR) );
	Anim.StopIncr(Anim.XACT_COMMIT);
	
	// commit xact1
	commit = MyShapes.XactCommit();
	commit.x = ExecLayout.COLUMN_START[i] + ExecLayout.XACT_GAP;
	commit.y = new_xact.y + new_xact_height - MyShapes.XACT_LINE_THICKNESS;

	commit.name = "commit"+i;
	addChild( commit );
	sp_commits.push( commit );
	w = Wiper.wipe( commit, Wiper.WIPE_RIGHT, true );
	g_timeline.addTween( new Tween(w.mask,"x",w.x,Anim.Start(),Anim.StopIncr(Anim.XACT_COMMIT),Transitions.LINEAR) );
},
function():void {
	sp_quanta = ExecLayout.removeByName( ["xact1b"], sp_quanta, removeChild );
	sp_commits = ExecLayout.removeByName( ["commit1"], sp_commits, removeChild );
	
	sp_quanta = ExecLayout.removeByName( ["xact2b"], sp_quanta, removeChild );
	sp_commits = ExecLayout.removeByName( ["commit2"], sp_commits, removeChild );
},
(2*Anim.XACT_COMMIT) + Anim.MOVE_TOKEN + (2*Anim.XACT_COMMIT));


// SHOW COMMUNICATION ARROWS
var sp_comm_arrows:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// dim code and CPU's
	for each ( var s:Sprite in sp_code_cpus.concat(sp_token,sp_time_arrows) ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,ExecLayout.ALPHA_DIM) );
	}
	Anim.StopIncr(Anim.FADE);
	
	// wipe in communication arrows
	//don't need RAW arrow - already have forwarding arrow
	var from:Sprite = ExecLayout.findByName( sp_insns, "i10" );
	var to:Sprite = ExecLayout.findByName( sp_insns, "i21" );
	var arrow:Sprite = new TextLineArrow( from.x+from.width-ExecLayout.COMM_ARROW_XOFFSET, from.y+(from.height/2), 
		to.x+ExecLayout.COMM_ARROW_XOFFSET, to.y+(to.height/2), "WAW", ExecLayout.COMM_ARROW_COLOR );
	addChild( arrow );
	sp_comm_arrows.push( arrow );
	
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(arrow, Anim.COMM_ARROW_WIPE) );
},
function():void {
	for each ( var s:Sprite in sp_comm_arrows ) { removeChild(s); }
	sp_comm_arrows = [];
},
(Anim.FADE) + (Anim.COMM_ARROW_WIPE));


// ADD CONCLUSION
var sp_conclusion:DisplayObject = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	var txt:TextField = new UnselectableTextField();
	txt.htmlText = "we've recovered nearly <b>all</b> parallelism\nwhile maintaining deterministic execution";
	txt.autoSize = TextFieldAutoSize.CENTER;
	var fmt:TextFormat = MyShapes.getLabelTextFormat();
	fmt.align = TextFormatAlign.CENTER;
	txt.setTextFormat( fmt );
	addChild( txt );
	txt.x = (ExecLayout.STAGE_WIDTH - txt.width) / 2;
	txt.y = 270;
	txt.alpha = 0;
	
	sp_conclusion = txt;
	g_timeline.addTween( ExecLayout.fadeToIncr(txt,1.0) );
},
function():void {
	removeChild( sp_conclusion );
	sp_conclusion = null;
},
Anim.FADE);


		SceneManager.noMoreScenes();
	} // end init()
	
}}